@@ -11,19 +11,27 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import contextlib
 from distutils import spawn
 import logging
 import os
 import re
 import select
 import signal
+import socket
 import subprocess
 import sys
+import tenacity
 import threading
+import time
 
 import fixtures
 import jinja2
 import six
+try:
+    import xattr
+except ImportError:
+    xattr = None
 
 if six.PY3:
     fsdecode = os.fsdecode
@@ -38,18 +46,20 @@ LOG = logging.getLogger(__name__)
 
 
 class Driver(fixtures.Fixture):
-    def __init__(self, env_prefix="PIFPAF", templatedir=".", debug=False):
+    def __init__(self, env_prefix="PIFPAF", templatedir=".", debug=False,
+                 tmp_rootdir=None):
         super(Driver, self).__init__()
         self.env_prefix = env_prefix
         self.env = {}
         self.debug = debug
+        self.tmp_rootdir = tmp_rootdir
 
         templatedir = os.path.join('drivers', 'templates', templatedir)
         self.template_env = jinja2.Environment(
             loader=jinja2.PackageLoader('pifpaf', templatedir))
 
     def _setUp(self):
-        self.tempdir = self.useFixture(fixtures.TempDir()).path
+        self.tempdir = self.useFixture(fixtures.TempDir(self.tmp_rootdir)).path
         self.putenv("DATA", self.tempdir)
 
     @staticmethod
@@ -62,13 +72,43 @@ class Driver(fixtures.Fixture):
         self.env[key] = value
         return self.useFixture(fixtures.EnvironmentVariable(key, value))
 
-    def _kill(self, pid, signal=signal.SIGTERM):
-        return os.kill(pid, signal)
+    def _ensure_xattr_support(self):
+        testfile = os.path.join(self.tempdir, "test")
+        self._touch(testfile)
+        xattr_supported = False
+        if xattr is not None:
+            try:
+                x = xattr.xattr(testfile)
+                x[b"user.test"] = b"test"
+            except (OSError, IOError) as e:
+                if e.errno != 95:
+                    raise
+            else:
+                xattr_supported = True
 
-    def _kill_pid_file(self, pidfile):
-        with open(pidfile, "r") as f:
-            pid = int(f.read().strip())
-        self._kill(pid)
+        if not xattr_supported:
+            raise RuntimeError("TMPDIR must support xattr for %s" %
+                               self.__class__.__name__)
+
+    def _kill(self, process):
+        pgrp = os.getpgid(process.pid)
+        process.terminate()
+        try:
+
+            self._wait(process)
+        except tenacity.RetryError:
+            LOG.warning("PID %d didn't terminate cleanly after 10 seconds, "
+                        "sending SIGKILL to its process group", process.pid)
+            # Cleanup remaining processes
+            os.killpg(pgrp, signal.SIGKILL)
+        process.wait()
+
+    @staticmethod
+    @tenacity.retry(wait=tenacity.wait_fixed(1),
+                    stop=tenacity.stop_after_attempt(10),
+                    retry=tenacity.retry_if_result(lambda ret: ret is None))
+    def _wait(process):
+        return process.poll()
 
     @staticmethod
     def find_executable(filename, extra_paths):
@@ -106,9 +146,10 @@ class Driver(fixtures.Fixture):
         LOG.debug("%s[%d] output: %s", appname, pid, data.rstrip())
 
     def _exec(self, command, stdout=False, ignore_failure=False,
-              stdin=None, wait_for_line=None, path=[], env=None,
+              stdin=None, wait_for_line=None, wait_for_port=None,
+              path=[], env=None,
               forbidden_line_after_start=None,
-              allow_debug=True, session=False):
+              allow_debug=True):
         LOG.debug("executing: %s" % command)
 
         app = command[0]
@@ -146,11 +187,11 @@ class Driver(fixtures.Fixture):
                 stdout=stdout_fd,
                 stderr=subprocess.STDOUT,
                 env=complete_env,
-                preexec_fn=os.setsid if session else None
+                preexec_fn=os.setsid,
             )
         except OSError as e:
             raise RuntimeError(
-                "Unable to run command `%s': %s" % (" ".join(command), e))
+                "Unable to run command `%s': %s" % (b" ".join(command), e))
 
         if stdin:
             LOG.debug("%s input: %s" % (app, stdin))
@@ -203,7 +244,19 @@ class Driver(fixtures.Fixture):
             t.setDaemon(True)
             t.start()
 
-        if not wait_for_line:
+        if wait_for_port:
+            for i in range(0, 10):
+                with contextlib.closing(
+                        socket.socket(socket.AF_INET,
+                                      socket.SOCK_STREAM)) as sock:
+                    if sock.connect_ex(('127.0.0.1', wait_for_port)) == 0:
+                        break
+                time.sleep(1)
+            else:
+                raise RuntimeError("Program did not open port %s" %
+                                   wait_for_port)
+
+        if not wait_for_line and not wait_for_port:
             status = c.wait()
             if not ignore_failure and status != 0:
                 raise RuntimeError("Error while running command: %s" % command)
