@@ -38,14 +38,14 @@ class ElasticsearchDriver(drivers.Driver):
         c, _ = self._exec(
             [
                 "elasticsearch",
-                "-Des.http.port=" + str(self.port),
-                "-Des.path.logs=%s" % os.path.join(self.tempdir, "log"),
-                "-Des.path.data=" + self.tempdir
+                "-Ehttp.port=" + str(self.port),
+                "-Epath.logs=%s" % os.path.join(self.tempdir, "log"),
+                "-Epath.data=" + self.tempdir
             ],
             path=["/usr/share/elasticsearch/bin"],
             wait_for_line=" started")
 
-        self.addCleanup(self._kill, c.pid)
+        self.addCleanup(self._kill, c)
 
         self.putenv("ELASTICSEARCH_PORT", str(self.port))
         self.putenv("URL", "es://localhost:%d" % self.port)
